<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use RecursiveDirectoryIterator;
use RecursiveFilterIterator;
use RecursiveIterator;
use RecursiveIteratorIterator;

/**
 * PathFilter
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Twig
 * @since   Infomaker\Everyware\Twig\PathFilter 0.1
 */
class PathFilter extends RecursiveFilterIterator
{

    /**
     * Contains an array of files and folders to exclude
     *
     * @var array
     */
    protected $exclude;

    /**
     * PathFilter constructor.
     *
     * @param \RecursiveIterator $iterator
     * @param array              $exclude
     */
    public function __construct(RecursiveIterator $iterator, array $exclude = [])
    {
        parent::__construct($iterator);
        $this->exclude = $exclude;
    }

    /**
     * Function to setup iterator from en absolute path
     *
     * @param string $path
     * @param array  $exclude
     *
     * @return static
     */
    public static function setupFromPath(string $path, array $exclude = []): PathFilter
    {
        return new static(new RecursiveDirectoryIterator($path), $exclude);
    }

    public function accept(): bool
    {
        if ($this instanceof RecursiveDirectoryIterator) {
            return ($this->isDir() && ! in_array($this->getFilename(), $this->exclude, true));
        }

        return false;
    }

    public function getChildren(): PathFilter
    {
        return new static(parent::getChildren(), $this->exclude);
    }

    /**
     * The filtered list in recursive order
     *
     * @return RecursiveIteratorIterator
     */
    public function filtered(): RecursiveIteratorIterator
    {
        return new RecursiveIteratorIterator($this);
    }
}
